class SessionsController < ApplicationController
  before_action :set_session, except: [:start_demo]
  before_action :set_schema_errors, only: %i[index own_creds_or_contract setup_contracting]
  before_action :set_contracting_resource, only: %i[index own_creds_or_contract check_existing_contract prepare_new_contract use_own_creds]
  before_action :check_auth, only: [:setup_contracting, :check_existing_contract, :prepare_new_contract]

  def index
    render_bad_request unless @session
    unless @contracting_resource
      return redirect_to(success_session_url(@session.id))
    end

    redirect_to(sessions_url(session_id: @session.id)) if @contracting_resource.reuse_previous_contracting
  end

  def start_demo
    begin
      @session = InitializeSession.call(git_params, params[:manual_resource_ids])
    rescue ACSmIExceptions::NoSchemaFound, ACSmIExceptions::InvalidSchema
      return redirect_to(failed_sessions_url)
    rescue StandardError => e
      return error_response(e.message)
    end

    redirect_to sessions_url(session_id: @session.id)
  end

  def own_creds_or_contract
    if params[:have_credentials] == 'true'
      @credential_fields = if @contracting_resource.resource_id
                             PlatformConnector.fields_for_credentials(@contracting_resource.resource_id)
                           else
                             [{'name' => 'login', 'type' => 'text'}, {'name' => 'password', 'type' => 'password'}]
                           end
      render 'request_credentials'
    else
      unless @contracting_resource.resource_id
        flash[:alert] = 'Corresponding CBP Resource is required for contracting'
        redirect_to sessions_url(session_id: @session.id)
        return false
      end

      if current_user
        @session.update(user: current_user)
        redirect_to setup_contracting_session_path(@session)
      else
        @user_session = UserSession.new
        @user = User.new
        @have_account = true
        render 'start_contracting'
      end
    end
  end

  def setup_contracting; end

  def success
    begin
      @session.write_to_repo
    rescue ACSmIExceptions::InvalidSchema
      return (render 'vm_failed')
    end
    @session.finish!
    render 'finish'
  end

  def check_existing_contract
    sleep 2
    first_contract = Contract.where(user_id: current_user.id, resource_id: @contracting_resource.resource_id).first
    @contract_exists = first_contract.present?
    if @contract_exists
      @contracting_resource.update(contract: first_contract)
      @contracting_resource.make_ready!
    end
  end

  def prepare_new_contract
    Contract.generate(@contracting_resource)
    sleep 1
    @contracting_resource.make_ready!
  end

  def use_own_creds
    @contracting_resource.save_credentials(params)
    redirect_to sessions_url(session_id: @session.id)
  end

  def hide_schema_warnings
    session[:hide_warning] ||= {}
    session[:hide_warning][@session.id] = true
    respond_to do |format|
      format.js { render nothing: true }
    end
  end

  private

  def git_params
    params.permit(%i[url username password])
  end

  def set_session
    @session = Session.find_by_id(params[:session_id]) || Session.find_by_id(params[:id])
  end

  def set_contracting_resource
    @contracting_resource = @session.contracting_resource
  end

  def contract_params
    params[:credential_fields]
  end

  def invalid_credentials
    flash[:alert] = "Couldn't find a contract with given credentials"
    redirect_to credentials_url
  end

  def set_schema_errors
    return true if session[:hide_warning].try(:[], @session.id)
    @schema_errors = ApplicationDescriptionSchema.user_friendly_errors @session.app_description.read
  end
end