class ContractingResource < ActiveRecord::Base
  belongs_to :session
  belongs_to :contract
  has_many :credential_fields
  has_one :virtual_machine

  include AASM

  aasm :status do
    state :waiting, initial: true
    state :completed
    state :completed_own
    state :failed

    event :make_ready do
      transitions from: :waiting, to: :completed
    end

    event :make_ready_own do
      transitions from: :waiting, to: :completed_own
    end

    event :fail do
      transitions from: :waiting, to: :failed
    end
  end

  def reuse_previous_contracting
    prev_res = session.contracting_resources.where(resource_id: resource_id, status: 'completed').first
    return false unless prev_res
    self.contract = prev_res.contract
    self.credential_fields = prev_res.credential_fields
    self.make_ready
    save
  end

  def save_credentials(params)
    params[:credential_fields].each_pair do |cf_name, cf_value|
      CredentialField.write(self, cf_name, cf_value)
    end
    self.make_ready_own!
  end

  def credential_fields_hash
    self.credential_fields.inject({}){|res, cf| res[cf.name] = cf.value; res}
  end

  def service
    Service.new(service_id)
  end

  def provider_name
    self.service.attribute('Provider', 'human')
  end

  def supports_redirection?
    # it should go to a separate module/class and be more smart
    # it is a fast track currently

    ['arsys', 'arsy'].include?(self.provider.downcase)
  end

  def redirect_url
    # this one as well should be separated
    return '' unless supports_redirection?

    'https://shop.arsys.es/client-data.php'
  end

  def use_default_data
    # is using hardcoded amazon and cs data
    # is needed for now, but is to be removed in future
    provider_value = self.provider.downcase
    if ['cloudsigma', 'clsi'].include? provider_value
      default_resource_id = 'c1fb1543-54aa-4247-91ee-3ea9a553f64e'
      default_resource_name = 'CloudSigma CloudBroker GmbH'
      default_region_id = '4265ddb9-e862-4814-82a4-d6b92f25e8e5'
      default_instance_type_id = '8c1ffcb7-4f50-42d7-8e88-bc7b8184b3cc'
    elsif ['amazon ec2', 'ams3', 'amdb', 'amazon', 'amaz'].include? provider_value
      default_resource_id = '05d76ee4-3b7e-4726-8ff3-eb24f667c70d'
      default_resource_name = 'Amazon EC2 CloudBroker GmbH'
      default_region_id = '83dc7c1e-711c-4548-9a9e-128f9795cfed'
      default_instance_type_id = 'e3ca8e4c-0f91-4e83-9bd9-4cef88d054a8'
    elsif ['arsys', 'arsy'].include? provider_value
      default_resource_id = '27b1fb7b-2fd9-4042-8902-e4996e0bc420'
      default_resource_name = 'Arsys CloudBuilder CloudBroker GmbH'
      default_region_id = 'd112253a-951f-417a-a1f1-3a892bf35a45'
      default_instance_type_id = 'f554e8a1-29d8-48e5-bd68-c01417576628'
    else
      raise "unsupported provider #{provider_value}"
    end

    self.resource_id = default_resource_id
    self.resource_name = default_resource_name
    self.build_virtual_machine(region_id: default_region_id,
                               instance_type_id: default_instance_type_id,
                               software_id: '21b7ebed-5076-43b6-8351-0e06cf16eedc',
                               image: 'Ubuntu 16.04')
  end

  def provider
    self.service.attribute('Provider')
  end

  def nice_name
    self.service_name ? "#{self.provider_name} (#{self.service_name})" : 'Not Found'
  end
end