package eu.decideh2020.acsmi.backend.services.server.repository;

import java.io.IOException;
import java.math.BigInteger;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.databind.ObjectMapper;

import eu.decideh2020.acsmi.backend.services.server.domain.Service;
import eu.decideh2020.acsmi.backend.services.server.domain.ServiceIncidence;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.AttributeVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.FilterAttributeVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.FilterServiceVO;

public class ServiceRepositoryImpl implements ServiceRepositoryCustom {
	
	private final Logger log = LoggerFactory.getLogger(ServiceRepositoryImpl.class);
	
	private final String NULL_VALUE = "null";
	private final String OPERATOR_GE = ">=";
	private final String OPERATOR_LE = "<=";
	private final String COMA_SEPARATOR = ",";
	
	private Map<Long, Service> matchMap;
	private List<Service> matchList;
	
	@PersistenceContext
    EntityManager entityManager;
	
	
	/**
	 * Discovers services.
	 * @param filter The filter.
	 * @return List
	 */
	@SuppressWarnings("unchecked")
	public List<Service> discover(String filter) throws Exception {
		log.debug("discover : finding Services...");
		List<Service> result = null;
		
		if (filter != null && !filter.isEmpty()) {
			ObjectMapper mapper = new ObjectMapper();
    		FilterServiceVO filterServiceVO = mapper.readValue(filter, FilterServiceVO.class);
    		if (filterServiceVO != null) {    			
    			String strQuery = buildQueryString(filterServiceVO.getClassid(), filterServiceVO.getAttributes());
    			if (strQuery != null) {
    				log.debug("discover : creating Query...  :  " + strQuery);
    				List<Service> serviceList = buildServiceList(entityManager.createNativeQuery(strQuery).getResultList());
    				if (filterServiceVO.getAttributes() == null || filterServiceVO.getAttributes().isEmpty()) {
    					// Solo se realiza la búsqueda por clase de servicio
    					return removeAttributesFromResult(serviceList);
    				} else {
    					// La búsqueda se realiza por atributos
    					if (serviceList != null && !serviceList.isEmpty()) {
    						return removeAttributesFromResult(match(filterServiceVO.getAttributes(), serviceList));
    					}
    				}
    			}
    		}
		}
		
		return result;
	}
	
	
	/**
	 * Build the query String.
	 * @param classid The classid
	 * @param attributes The attribute list
	 * @return String
	 */
	private String buildQueryString(final Long classid, final List<FilterAttributeVO> attributes) {
		// Resolve typeid's
		StringBuffer typeids = null;
		if (attributes != null && !attributes.isEmpty()) {
			typeids = new StringBuffer("");
			final int len = attributes.size();
			for (int i = 0;i < len;i++) {
				typeids.append(attributes.get(i).getTypeid());
				if (attributes.get(i).getChildid() != null) {
					typeids.append(",").append(attributes.get(i).getChildid());
				}
				if (i < len - 1) {
					typeids.append(",");
				}
			}
		}
		
		// Service ([0-4]) and ServiceClass ([5]) properties
		StringBuffer strQuery = new StringBuffer("SELECT s.id, s.servicename, ");
		strQuery.append("s.serviceclassid, s.created_date, s.last_modified_date, sc.serviceclassname, ");
		// ServiceAttributeType and ServiceAttributeValue ([6-23]) properties
		strQuery.append("sat.id as satid, sat.name, sat.nfrname, sat.isenumeration, sat.ischecklist, ");
		strQuery.append("sat.iscommon, sat.isfunctionalrequirement, sat.parentid, sat.childid, sat.units, ");
		strQuery.append("sat.unitfactor, sat.unitrule, sat.checklistevalrule, sav.unitvalue, sav.checklistvalue, ");
		strQuery.append("sav.id as savid, sav.serviceattributevalue, d.value, ");
		// ServiceIncidence properties ([24-29])
		strQuery.append("si.id as incidenceid, si.timestamp, si.detail, si.serviceid, si.created_date as indicencecd, ");
		strQuery.append("si.last_modified_date as incidencelmd ");
		strQuery.append("FROM service s ");
		strQuery.append("JOIN service_class sc on s.serviceclassid = sc.id ");
		strQuery.append("JOIN service_attribute_value sav on s.id = sav.serviceid ");
		strQuery.append("JOIN service_attribute_type sat on sat.id = sav.serviceattributetypeid ");
		strQuery.append("LEFT OUTER JOIN definition d on sav.serviceattributevalue = d.code ");
		strQuery.append("LEFT OUTER JOIN service_incidence si on s.id = si.serviceid ");
		strQuery.append("WHERE s.serviceclassid = ").append(classid);
		strQuery.append(" AND s.deleted_date IS NULL ORDER BY s.servicename, sat.id, si.id");
		
		return strQuery != null ? strQuery.toString() : null;
	}
	
	
	/**
	 * Builds a Service list from an Object[] list.
	 * @param objectList The Object[] list.
	 * @return List
	 */
	private List<Service> buildServiceList(final List<Object[]> objectList) throws IOException {
		Map<Long, Service> map = new HashMap<Long, Service>();
		Map<String, ServiceIncidence> incidences = new HashMap<String, ServiceIncidence>();
		Map<String, AttributeVO> attributes = new HashMap<String, AttributeVO>();
		if (objectList != null && !objectList.isEmpty()) {
			for (Object[] array : objectList) {
				// Build Service object and add it to the map
				Service srv = buildServiceObject(array);
				map.put(srv.getId(), srv);
				
				// Add ServiceIncidence to the map
				if (array[24] != null) {
					incidences.put(srv.getId() + "," + String.valueOf(array[24]), buildServiceIncidenceObject(array));
				}
				
				// Add AttributeVO to the map
				attributes.put(srv.getId() + "," + String.valueOf(array[6]), buildAttributeVOObject(array));
			}
		}
		
		List<Service> result = new ArrayList<Service>(map.values());
		for (Service srv : result) {
			// Load incidences
			for (String key : incidences.keySet()) {
				if (key.startsWith(srv.getId() + ",")) {
					srv.getIncidences().add(incidences.get(key));
				}
			}
			
			// Load attributes
			for (String key : attributes.keySet()) {
				if (key.startsWith(srv.getId() + ",")) {
					srv.getAttributes().add(attributes.get(key));
				}
			}
		}
		return result;
	}
	
	
	/**
	 * Builds a Service object.
	 * @param object The object array.
	 * @return Service
	 */
	private Service buildServiceObject(final Object[] object) {
		Service service = new Service();
		service.setId(new BigInteger(String.valueOf(object[0])).longValue());
		service.setServicename(String.valueOf(object[1]));
		service.setServiceclassid(new BigInteger(String.valueOf(object[2])).longValue());
		service.setCreatedDate(object[3] == null ? null : ((Timestamp)object[3]).toInstant());
		service.setLastModifiedDate(object[4] == null ? null : ((Timestamp)object[4]).toInstant());
		service.setServiceclassname(String.valueOf(object[5]));
		return service;
	}
		
	
	/**
	 * Builds an AttributeVO object.
	 * @param object The object array.
	 * @return AttributeVO
	 */
	private AttributeVO buildAttributeVOObject(final Object[] object) {
		AttributeVO attributeVO = new AttributeVO();
		attributeVO.setTypeid(new BigInteger(String.valueOf(object[6])).longValue());
		attributeVO.setName(String.valueOf(object[7]));
		attributeVO.setNfrname(object[8] == null ? null : String.valueOf(object[8]));
		attributeVO.setIsenumeration(new Boolean(String.valueOf(object[9])));
		attributeVO.setIschecklist(new Boolean(String.valueOf(object[10])));
		attributeVO.setIscommon(new Boolean(String.valueOf(object[11])));
		attributeVO.setIsfunctionalrequirement(new Boolean(String.valueOf(object[12])));
		attributeVO.setParentid(object[13] == null ? null : new BigInteger(String.valueOf(object[13])).longValue());
		attributeVO.setChildid(object[14] == null ? null : new BigInteger(String.valueOf(object[14])).longValue());
		attributeVO.setUnits(object[15] == null ? null : String.valueOf(object[15]));
		attributeVO.setUnitfactor(object[16] == null ? null : String.valueOf(object[16]));
		attributeVO.setUnitrule(object[17] == null ? null : String.valueOf(object[17]));
		attributeVO.setChecklistevalrule(object[18] == null ? null : String.valueOf(object[18]));
		attributeVO.setUnitvalue(object[19] == null ? null : String.valueOf(object[19]));
		attributeVO.setChecklistvalue(object[20] == null ? null : String.valueOf(object[20]));
		attributeVO.setValueid(new BigInteger(String.valueOf(object[21])).longValue());
		final String value = object[23] != null && !String.valueOf(object[23]).trim().equalsIgnoreCase(NULL_VALUE)
				? String.valueOf(object[23]) : String.valueOf(object[22]);
		attributeVO.setValue(value);
		attributeVO.setHuman(object[23] == null ? null : String.valueOf(object[23]));	// Definition
		return attributeVO;
	}
	
	
	/**
	 * Builds a ServiceIncidence object.
	 * @param object The object array.
	 * @return ServiceIncidence
	 */
	private ServiceIncidence buildServiceIncidenceObject(final Object[] object) {
		return new ServiceIncidence(
				new BigInteger(String.valueOf(object[24])).longValue(),
				new BigInteger(String.valueOf(object[25])).longValue(),
				String.valueOf(object[26]),
				new BigInteger(String.valueOf(object[27])).longValue(),
				((Timestamp)object[28]).toInstant(),
				((Timestamp)object[29]).toInstant());
	}
	
	
	/**
	 * Matches the filter and the result.
	 * @param attributes Filter attributes.
	 * @param services The services.
	 * @return List
	 */
	private List<Service> match(final List<FilterAttributeVO> attributes, final List<Service> services) {
		matchMap = new HashMap<Long, Service>();
		matchList = new ArrayList<Service>();
		
		// Cotejar datos
		for (FilterAttributeVO attr : attributes) {
			for (Service srv : services) {
				final List<AttributeVO> attributeList = srv.getAttributes();
				for (AttributeVO attrSrv : attributeList) {
					if (attr.getTypeid().longValue() == attrSrv.getTypeid().longValue()
							|| (attr.getChildid() != null && attr.getChildid().longValue() == attrSrv.getTypeid().longValue())) {
						if (attrSrv.getUnits() != null && !attrSrv.getUnits().isEmpty()) {
							// Preparar los valores a comparar
							double[] values = convertValues(attr, attrSrv);

							// Comparar valores
							if (attrSrv.getUnitrule().equalsIgnoreCase(OPERATOR_GE) && values[1] >= values[0]) {
								srv.getMatchingattributelist().add(attrSrv.getName());
								buildMatch(srv, attrSrv.getName());
							} else if (attrSrv.getUnitrule().equalsIgnoreCase(OPERATOR_LE) && values[1] <= values[0]) {
								srv.getMatchingattributelist().add(attrSrv.getName());
								buildMatch(srv, attrSrv.getName());
							}
						}
						else if (attrSrv.getValue().toLowerCase().indexOf(attr.getValue().toLowerCase()) != -1) {
							srv.getMatchingattributelist().add(attrSrv.getName());
							buildMatch(srv, attrSrv.getName());
						}
					}
				}
			}
		}
		
		// Calcular % de acierto
		final double numCriteria = (double)attributes.size();
		for (Map.Entry<Long, Service> entry : matchMap.entrySet()) {
			double counter = 0D;
			for (Service srv : matchList) {
				if (entry.getKey().longValue() == srv.getId().longValue()) {
					counter++;
				}
			}
			final double matchResult = numCriteria == 0 ? 100D : Math.round(((counter / numCriteria) * 100D) * 100.0) / 100.0;
			entry.getValue().setMatch(matchResult);
		}
		
		return new ArrayList<Service>(matchMap.values());
	}
	
	
	/**
	 * Convert the values of the filter and the VO stored to evaluate later
	 * @param attr The filter
	 * @param vo The VO stored
	 * @return double[]
	 */
	private double[] convertValues(final FilterAttributeVO attr, final AttributeVO vo) {
		double[] result = { 0d, 0d };
		if (attr.getUnit().equalsIgnoreCase(vo.getUnitvalue())) {
			result[0] = Double.valueOf(attr.getValue()).doubleValue();
			if (vo.getValue() != null && !vo.getValue().equals(NULL_VALUE)) {
				result[1] = Double.valueOf(vo.getValue()).doubleValue();
			}
		} else {
			final String[] arrayunits = vo.getUnits().split(COMA_SEPARATOR);
			final String[] arrayfactors = vo.getUnitfactor().split(COMA_SEPARATOR);
			final int[] indexes = resolveIndexes(arrayunits, attr.getUnit(), vo.getUnitvalue());
			result[0] = Double.valueOf(attr.getValue()).doubleValue() *
					(Double.valueOf(arrayfactors[0]).doubleValue() / Double.valueOf(arrayfactors[indexes[0]]).doubleValue());
			if (vo.getValue() != null && !vo.getValue().equals(NULL_VALUE)) {
				result[1] = Double.valueOf(vo.getValue()).doubleValue() *
						(Double.valueOf(arrayfactors[0]).doubleValue() / Double.valueOf(arrayfactors[indexes[1]]).doubleValue());
			}
		}
		return result;
	}
	
	
	/**
	 * Builds a match.
	 * @param service The service.
	 * @param attributetypename The name of the attribute type.
	 */
	private void buildMatch(final Service service, final String attributetypename) {
		matchList.add(service);
		if (!matchMap.containsKey(service.getId())) {
			matchMap.put(service.getId(), service);
		}
	}
	
	
	/**
	 * Resolve the indexes of units and factors
	 * @param arrayunits Array of units.
	 * @param filterUnit The unit in the filter.
	 * @param voUnit The unit in the VO.
	 * @return int[]
	 */
	private int[] resolveIndexes(final String[] arrayunits, final String filterUnit, final String voUnit) {
		int[] result = { 0, 0 };
		for (int i = 0;i < arrayunits.length;i++) {
			if (arrayunits[i].equalsIgnoreCase(filterUnit)) {
				result[0] = i;
			}
			if (arrayunits[i].equalsIgnoreCase(voUnit)) {
				result[1] = i;
			}
		}
		return result;
	}
	
	
	private List<Service> removeAttributesFromResult(final List<Service> list) {
		if (list != null && !list.isEmpty()) {
			for (Service srv : list) {
				srv.getAttributes().clear();
			}
		}
		return list;
	}
}