/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;

import eu.decideh2020.acsmi.backend.services.server.domain.ServiceAttributeType;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceAttributeTypeRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing ServiceAttributeType.
 */
@RestController
@RequestMapping("/api")
public class ServiceAttributeTypeResource {

    private final Logger log = LoggerFactory.getLogger(ServiceAttributeTypeResource.class);

    private static final String ENTITY_NAME = "serviceAttributeType";
        
    private final ServiceAttributeTypeRepository serviceAttributeTypeRepository;

    public ServiceAttributeTypeResource(ServiceAttributeTypeRepository serviceAttributeTypeRepository) {
        this.serviceAttributeTypeRepository = serviceAttributeTypeRepository;
    }
    

    /**
     * GET  /service-attribute-types : get all the ServiceAttributeTypes.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of ServiceAttributeTypes in body
     */
    @GetMapping("/service-attribute-types")
    @Timed
    public ResponseEntity<List<ServiceAttributeType>> findServiceAttributeTypes(@RequestParam String classid) {
    	log.debug("REST request to find all ServiceAttributeTypes of the classid :: " + classid);
    	try {
    		List<ServiceAttributeType> list = null;
        	if (classid == null || classid.trim().length() == 0) {
        		log.debug("findServiceAttributeTypes  ::  getting ServiceAttributeTypes...");
        		list = serviceAttributeTypeRepository.findAll();
        	} else {
        		log.debug("findServiceAttributeTypes  ::  getting ServiceAttributeTypes of the classid  ::  " + classid);
        		List<Long> excluded = findChildAttributes();
        		list = serviceAttributeTypeRepository.findServiceAttributeTypesByClassidExcludingAttributes(
        				new Long(classid), excluded);
        	}
        	return ResponseEntity.ok(list);
    	} catch (Exception e) {
    		log.error("findServiceAttributeTypes  ::  Exception  ::  " + e.getMessage());
    		return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceAttributeTypeGetAllError", "Cannot get the list of Service attribute types")).body(null);
    	}
    }

    /**
     * GET  /service-attribute-types/:id : get the "id" ServiceAttributeTypes.
     *
     * @param id the id of the ServiceAttributeTypes to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the ServiceAttributeTypes, or with status 404 (Not Found)
     */
    @GetMapping("/service-attribute-types/{id}")
    @Timed
    public ResponseEntity<ServiceAttributeType> getServiceAttributeType(@PathVariable Long id) {
        log.debug("REST request to get ServiceAttributeType : {}", id);
        ServiceAttributeType serviceAttributeType = null;
        try {
        	log.debug("getServiceAttributeType  ::  getting ServiceAttributeType...");
        	serviceAttributeType = serviceAttributeTypeRepository.findOne(id);
        	
        	log.debug("getServiceAttributeType  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(serviceAttributeType));
        } catch (Exception e) {
        	log.error("getServiceAttributeType  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceAttributeTypeGetError", "Cannot get the Service attribute type"))
                    .body(null);
        }
    }
    
    
    /**
     * Finds the child attributes.
     * @return List
     */
    private List<Long> findChildAttributes() {
    	List<Long> result = new ArrayList<Long>();
    	try {
    		log.debug("findChildAttributes  ::  getting child attributes...");
    		List<ServiceAttributeType> list = serviceAttributeTypeRepository.findChildAttributes();
    		if (list != null && !list.isEmpty()) {
    			for (ServiceAttributeType attr : list) {
    				result.add(attr.getId());
    			}
    		}
    	} catch (Exception e) {
    		log.error("findChildAttributes  ::  Exception  ::  " + e.getMessage());
    	}
    	return result;
    }
}
