/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;

import eu.decideh2020.acsmi.backend.services.server.domain.ServiceClass;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.AttributeVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceClassFullInfoVO;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceClassRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing ServiceClass.
 */
@RestController
@RequestMapping("/api")
public class ServiceClassResource {

    private final Logger log = LoggerFactory.getLogger(ServiceClassResource.class);

    private static final String ENTITY_NAME = "serviceClass";
        
    private final ServiceClassRepository serviceClassRepository;

    public ServiceClassResource(ServiceClassRepository serviceClassRepository) {
        this.serviceClassRepository = serviceClassRepository;
    }

    
    /**
     * GET  /service-classes : get all the serviceClasses.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of serviceClasses in body
     */
    @GetMapping("/service-classes")
    @Timed
    @Transactional
    public List<ServiceClass> getAllServiceClasses() {
       log.debug("REST request to get all ServiceClasses");
        try {log.debug("getAllServiceClasses  ::  getting all Service Classes...");
        	return serviceClassRepository.findAll();
        } catch (Exception e) {
        	log.error("getAllServiceClasses  ::  Exception  ::  " + e.getMessage());
        	return null;
        }
    }
    
    /**
     * GET  /service-classes/full : get all the serviceClasses (full info)
     *
     * @return the ResponseEntity with status 200 (OK) and the list of serviceClasses in body
     */
    @GetMapping("/service-classes/full")
    @Timed
    @Transactional
    public ResponseEntity<List<ServiceClass>> getAllServiceClassesFullInfo() {
       log.debug("REST request to get all ServiceClasses");
        try {
        	log.debug("getAllServiceClassesFullInfo  ::  getting all Service Classes...");
        
	        List<ServiceClass> result = null;
	    	List<ServiceClassFullInfoVO> list = serviceClassRepository.findServiceClasses();
	    	if (list != null && !list.isEmpty()) {
	    		result = new ArrayList<ServiceClass>();
	    		
	    		long currentclassid = 0;
	    		ServiceClass serviceclass = new ServiceClass();
	    		for (ServiceClassFullInfoVO vo : list) {
	    			if (vo.getId().longValue() != currentclassid) {
	    				serviceclass = new ServiceClass();
	    				serviceclass.setId(vo.getId());
		    			serviceclass.setServiceclassname(vo.getName());
	    			}
	    			
	    			AttributeVO attr = buildAttributeVO(vo);
		    		if (vo.isIscommon()) {
		    			serviceclass.getCommonattributes().add(attr);
		    		} else if (vo.isIsfunctionalrequirement()) {
		    			serviceclass.getFunctionalattributes().add(attr);
		    		} else {
		    			serviceclass.getNonfunctionalattributes().add(attr);
		    		}
	    			
	    			if (vo.getId().longValue() != currentclassid) {
	    				currentclassid = vo.getId().longValue();
	    				result.add(serviceclass);
	    			}
	    		}
	    	}
	    	return ResponseEntity.ok(result);
        } catch (Exception e) {
        	log.error("getAllServiceClassesFullInfo  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceClassGetAllError", "Cannot get the list of Service classes")).body(null);
        }
    }

    /**
     * GET  /service-classes/:id : get the "id" serviceClass.
     *
     * @param id the id of the serviceClass to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the serviceClass, or with status 404 (Not Found)
     */
    @GetMapping("/service-classes/{id}")
    @Timed
    public ResponseEntity<ServiceClass> getServiceClass(@PathVariable Long id) {
        log.debug("REST request to get ServiceClass : {}", id);
        ServiceClass serviceClass = null;
        try {
        	log.debug("getServiceClass  ::  getting service class...");
        	serviceClass = serviceClassRepository.findOne(id);
        	log.debug("getServiceClass  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(serviceClass));
        } catch (Exception e) {
        	log.error("getServiceClass  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceClassGetError", "No puede obtenerse la clase de servicio"))
                    .body(null);
        }
    }
    
    
    /**
     * Builds an AttributeVO object.
     * @param object ServiceClassFullInfoVO object.
     * @return AttributeVO
     */
    private AttributeVO buildAttributeVO(final ServiceClassFullInfoVO object) {
    	AttributeVO attr = new AttributeVO();
		attr.setTypeid(object.getAttributetypeid());
		attr.setName(object.getAttributename());
		attr.setNfrname(object.getAttributenfrname());
		attr.setIsenumeration(object.isIsenumeration());
		attr.setIschecklist(object.isIschecklist());
		attr.setIscommon(object.isIscommon());
		attr.setIsfunctionalrequirement(object.isIsfunctionalrequirement());
		attr.setParentid(object.getParentid());
		attr.setChildid(object.getChildid());
		attr.setUnits(object.getUnits());
		attr.setChecklistevalrule(object.getChecklistevalrule());
		attr.setUnitvalue(object.getUnitvalue());
		attr.setUnitfactor(object.getUnitfactor());
		attr.setUnitrule(object.getUnitrule());
		attr.setChecklistvalue(object.getChecklistvalue());
		attr.setClassattributeid(object.getClassattributeid());
		attr.setWeight(object.getWeight());
		attr.setIsmandatory(object.isIsmandatory());
		return attr;
    }
}
