/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;

import eu.decideh2020.acsmi.backend.services.server.domain.ServiceIncidence;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceIncidenceRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;

/**
 * REST controller for managing ServiceIncidence.
 */
@RestController
@RequestMapping("/api")
public class ServiceIncidenceResource {

    private final Logger log = LoggerFactory.getLogger(ServiceIncidenceResource.class);

    private static final String ENTITY_NAME = "serviceIncidence";
    
    private final ServiceIncidenceRepository serviceIncidenceRepository;

    
    public ServiceIncidenceResource(ServiceIncidenceRepository serviceIncidenceRepository) {
        this.serviceIncidenceRepository = serviceIncidenceRepository;
    }

    /**
     * POST  /service-incidences : Create a new serviceIncidence.
     *
     * @param serviceIncidence the serviceIncidence to create
     * @return the ResponseEntity with status 201 (Created) and with body the new serviceIncidence, or with status 400 (Bad Request) if the serviceIncidence has already an ID
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PostMapping("/service-incidences")
    @Timed
    @Transactional
    public ResponseEntity<ServiceIncidence> createServiceIncidence(@Valid @RequestBody ServiceIncidence serviceIncidence) throws URISyntaxException {
    	log.debug("REST request to save ServiceIncidence : {}", serviceIncidence);
        try {        	
        	if (serviceIncidence.getId() != null) {
                return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(
                		ENTITY_NAME, "idexists", "A new serviceIncidence cannot already have an ID")).body(null);
            }
        	
        	// Insertar la incidencia de servicio
        	log.debug("createServiceIncidence  ::  creating Service...");
        	ServiceIncidence result = serviceIncidenceRepository.saveAndFlush(serviceIncidence);
        	log.debug("createServiceIncidence  ::  creating OK");
        	
            return ResponseEntity.created(new URI("/api/service-incidences/" + result.getId()))
                .headers(HeaderUtil.createEntityCreationAlert(ENTITY_NAME, result.getDetail()))
                .body(result);
        } catch (Exception e) {
        	log.error("createServiceIncidence  ::  Exception  ::  " + e.getMessage());
        	e.printStackTrace();
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceIncidenceCreateError", "Cannot create the Service incidence"))
                    .body(null);
        }
    }

    /**
     * GET  /service-incidences/:id : get all the service incidences.
     * @param id the id of the service
     *
     * @return the ResponseEntity with status 200 (OK) and the list of service incidences in body
     */
    @GetMapping("/service-incidences/{id}")
    @Timed
    public ResponseEntity<List<ServiceIncidence>> getAllServiceIncidences(@PathVariable Long id) {
    	log.debug("REST request to get all Service incidences");
        try {
        	log.debug("getAllServiceIncidence  ::  getting all Service incidences...");
        	return ResponseEntity.ok(findIncidencesByServiceId(id));
        } catch (Exception e) {
        	log.error("getAllServiceIncidence  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceIncidenceGetAllError", "Cannot get the list of Service incidences")).body(null);
        }
    }
    
    
    /**
     * Find the incidences for a service id
     * @param serviceid The serviceid
     * @return List
     */
    public List<ServiceIncidence> findIncidencesByServiceId(final Long serviceid) {
    	return serviceIncidenceRepository.findIncidencesByServiceId(serviceid);
    }
}
