/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import eu.decideh2020.acsmi.backend.services.server.domain.Definition;
import eu.decideh2020.acsmi.backend.services.server.domain.Service;
import eu.decideh2020.acsmi.backend.services.server.domain.ServiceAttributeValue;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.AttributeVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceDevOpsInfoVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceDevOpsQueryInfoVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceFullInfoVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceNFRInfoExtendedVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceNFRInfoVO;
import eu.decideh2020.acsmi.backend.services.server.repository.DefinitionRepository;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceAttributeValueRepository;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceIncidenceRepository;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing Service.
 */
@RestController
@RequestMapping("/api")
public class ServiceResource {

    private final Logger log = LoggerFactory.getLogger(ServiceResource.class);

    private static final String ENTITY_NAME = "service";
    
    private final ServiceRepository serviceRepository;
    private final ServiceAttributeValueRepository serviceAttributeValueRepository;
    private final DefinitionRepository definitionRepository;
    private final ServiceIncidenceRepository serviceIncidenceRepository;
    
    private final Long CSP_TYPE_ID = 3L;

    
    public ServiceResource(ServiceRepository serviceRepository, ServiceAttributeValueRepository serviceAttributeValueRepository,
    		DefinitionRepository definitionRepository, ServiceIncidenceRepository serviceIncidenceRepository) {
        this.serviceRepository = serviceRepository;
        this.serviceAttributeValueRepository = serviceAttributeValueRepository;
        this.definitionRepository = definitionRepository;
        this.serviceIncidenceRepository = serviceIncidenceRepository;
    }

    /**
     * POST  /services : Create a new service.
     *
     * @param service the service to create
     * @return the ResponseEntity with status 201 (Created) and with body the new service, or with status 400 (Bad Request) if the service has already an ID
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PostMapping("/services")
    @Timed
    @Transactional
    public ResponseEntity<Service> createService(@Valid @RequestBody Service service, @RequestParam List<String> attributes) throws URISyntaxException {
    	log.debug("REST request to save Service : {}", service);
        try {        	
        	if (service.getId() != null) {
                return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(
                		ENTITY_NAME, "idexists", "A new service cannot already have an ID")).body(null);
            }
        	
        	// Comprobar que el nombre del servicio no exista previamente
        	if (checkIfServiceExistsForInsert(service.getServicename())) {
        		return ResponseEntity.badRequest()
                        .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceNameExists", "The Service name is already registered"))
                        .body(null);
        	}
        	
        	// Insertar el servicio
        	log.debug("createService  ::  creating Service...");
        	Service result = serviceRepository.saveAndFlush(service);
        	
        	// Insertar los valores de atributos
        	if (attributes != null && !attributes.isEmpty()) {
    			ServiceAttributeValueResource attrResource = new ServiceAttributeValueResource(serviceAttributeValueRepository);
        		for (String attr : attributes) {
        			ServiceAttributeValue attribute = convertAttributeStringToObject(result.getId(), attr);
        			attrResource.createServiceAttributeValue(attribute);
        		}
    		}
        	log.debug("createService  ::  creating OK");
        	
            return ResponseEntity.created(new URI("/api/services/" + result.getId()))
                .headers(HeaderUtil.createEntityCreationAlert(ENTITY_NAME, result.getServicename()))
                .body(result);
        } catch (Exception e) {
        	log.error("createService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceCreateError", "No puede crearse el Servicio"))
                    .body(null);
        }
    }

    /**
     * PUT  /services : Updates an existing service.
     *
     * @param service the service to update
     * @return the ResponseEntity with status 200 (OK) and with body the updated service,
     * or with status 400 (Bad Request) if the service is not valid,
     * or with status 500 (Internal Server Error) if the service couldnt be updated
     * @throws URISyntaxException if the Location URI syntax is incorrect
     */
    @PutMapping("/services")
    @Timed
    @Transactional
    public ResponseEntity<Service> updateService(@Valid @RequestBody Service service, @RequestParam List<String> attributes) throws URISyntaxException {
    	log.debug("REST request to update Service : {}", service);
        try {
        	boolean reactivate = false;
        	
        	// Si la fecha de baja no es nula, se trata de una reactivación
        	if (service.getDeleteddate() != null) {
        		log.debug("updateService  ::  reactivating Service...");
        		reactivate = true;
        		
        		// Reactivar atributos del servicio
	        	serviceAttributeValueRepository.updateDeletedDateByIdService(null, service.getId());
	        	
        		service.setDeleteddate(null);
        	} else {
        		if (service.getId() == null) {
                    return createService(service, attributes);
                }
        		
        		// Comprobar que el servicio no existe previamente
        		if (checkIfServiceExistsForUpdate(service.getId(), service.getServicename())) {
            		return ResponseEntity.badRequest()
                            .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceNameExists", "The Service name is already registered"))
                            .body(null);
            	}
        		
        		// Tratar atributos
        		if (attributes != null && !attributes.isEmpty()) {
        			ServiceAttributeValueResource attrResource = new ServiceAttributeValueResource(serviceAttributeValueRepository);
        			
        			// Eliminar atributos y posteriormente insertarlos
            		serviceAttributeValueRepository.deleteByIdService(service.getId());
            		
            		// Insertar los valores de atributos
            		for (String attr : attributes) {
            			ServiceAttributeValue attribute = convertAttributeStringToObject(service.getId(), attr);
            			attrResource.createServiceAttributeValue(attribute);
            		}
        		}
        	}
        	
        	// Modificar el servicio
        	log.debug("updateService  ::  updating Service...");
        	Service result = serviceRepository.saveAndFlush(service);
        	log.debug("updateService  ::  updating OK");
        	
            return reactivate
            		? ResponseEntity.ok().headers(HeaderUtil.createEntityReactivateAlert(ENTITY_NAME, service.getServicename())).body(result)
            				: ResponseEntity.ok().headers(HeaderUtil.createEntityUpdateAlert(ENTITY_NAME, service.getServicename())).body(result);
        } catch (Exception e) {
        	log.error("updateService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceUpdateError", "No puede actualizarse el Servicio"))
                    .body(null);
        }
    }

    /**
     * GET  /services : get all the services.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of services in body
     */
    @GetMapping("/services")
    @Timed
    public ResponseEntity<List<Service>> getAllServices() {
    	log.debug("REST request to get all Services");
        try {
        	// Obtener lista de servicios
        	log.debug("getAllServices  ::  getting all Services...");
        	List<Service> list = serviceRepository.findAllServices();
        	if (list != null && !list.isEmpty()) {
        		list = inlineServiceIncidences(list);
        	}
        	return ResponseEntity.ok(list);
        } catch (Exception e) {
        	log.error("getAllServices  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceGetAllError", "Cannot get the list of Services")).body(null);
        }
    }
    
    /**
     * GET  /services/find : get all the services.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of services in body
     */
    @GetMapping("/services/find")
    @Timed
    public ResponseEntity<List<Service>> findServices(@RequestParam String filter) {
    	log.debug("REST request to get all Services");
        try {
        	return ResponseEntity.ok(serviceRepository.discover(filter));
        } catch (Exception e) {
        	log.error("findServices  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceFindError", "Cannot find the Services")).body(null);
        }
    }

    
    /**
     * GET  /service-providers/devops : get all the serviceProviderss.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of serviceProviderss in body
     */
    @GetMapping("/services/devops")
    @Timed
    @Transactional
    public ResponseEntity<List<ServiceDevOpsInfoVO>> getDevOpsInfo() {
       log.debug("REST request to get DevOps info");
        try {
        	log.debug("getDevOpsInfo  ::  getting DevOps info...");
        	
        	List<ServiceDevOpsInfoVO> result = new ArrayList<ServiceDevOpsInfoVO>();
        	List<String> keylist = serviceRepository.getDevOpsKeys();
        	
        	if (keylist != null && !keylist.isEmpty()) {
        		List<ServiceDevOpsQueryInfoVO> querylist = serviceRepository.getDevOpsQueryInfo();
        		if (querylist != null && !querylist.isEmpty()) {
        			for (String str : keylist) {
        				int counter = 0;
           				for (ServiceDevOpsQueryInfoVO vo : querylist) {
           					if (str.equalsIgnoreCase(vo.getQuerykey())) {
           						counter++;
           					}
           				}
            			
           				log.debug("getDevOpsInfo  ::  adding to result...");
           				String[] keys = str.split(";");
               			result.add(new ServiceDevOpsInfoVO(keys[0], keys[1], counter));
            		}
        		}
        	}
        	return ResponseEntity.ok(result);
        } catch (Exception e) {
        	log.error("getDevOpsInfo  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceGetDevOpsInfoError", "Cannot get the Service DevOps Info")).body(null);
        }
    }
    
    
    /**
     * GET  /services/nfr : get the NFR info
     *
     * @return the ResponseEntity with status 200 (OK) and the NFR info in body
     */
    @GetMapping("/services/nfr")
    @Timed
    public ResponseEntity<List<ServiceNFRInfoVO>> getServiceNfrInfo(@RequestParam List<Long> serviceids) {
    	log.debug("REST request to get the NFR info");
    	try {
    		// Cargar mapa de definiciones
    		log.debug("getServiceNfrInfo  ::  loading Definitions...");
    		DefinitionResource definitionResource = new DefinitionResource(definitionRepository);
    		List<Definition> definitionList = definitionResource.getAllDefinitions().getBody();
    		log.debug("getServiceNfrInfo  ::  Definitions loaded");
    		Map<String, String> map = DefinitionResource.getDefinitionMap(definitionList);
    		
    		List<ServiceNFRInfoVO> result = new ArrayList<ServiceNFRInfoVO>();
    		if (serviceids != null && !serviceids.isEmpty()) {
    			log.debug("getServiceNfrInfo  ::  loading NFR's...");
    			List<ServiceNFRInfoExtendedVO> list = serviceRepository.getNFRInfo(serviceids);
            	if (list != null && !list.isEmpty()) {
            		for (ServiceNFRInfoExtendedVO vo : list) {
            			log.debug("getServiceNfrInfo  ::  build ServiceNFRInfoVO object...");
            			ServiceNFRInfoVO infoVO = new ServiceNFRInfoVO();
            			infoVO.setServiceid(vo.getServiceid());
            			infoVO.setServicename(vo.getServicename());
            			infoVO.setServiceclassname(vo.getServiceclassname());
            			infoVO.setNfrname(vo.getNfrname());
            			infoVO.setNfrmetric(vo.getNfrmetric());
            			infoVO.setNfrvalue(vo.isIsenumeration() ? map.get(vo.getNfrvalue()) : vo.getNfrvalue());
            			result.add(infoVO);
            		}
            	}
    		}
    		return ResponseEntity.ok(result);
        } catch (Exception e) {
        	log.error("getServiceNfrInfo  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceGetNFRError", "Cannot get the Service NFR Info")).body(null);
        }
    }
    
    
    /**
     * GET  /services/optimus : get the Optimus info
     *
     * @return the ResponseEntity with status 200 (OK) and the Optimus info in body
     */
    @GetMapping("/services/optimus")
    @Timed
    public ResponseEntity<List<Service>> getOptimusInfo(@RequestParam List<Long> serviceids) {
    	log.debug("REST request to get the Optimus info");
    	try {
    		log.debug("getOptimusInfo  ::  loading Optimus info...");
    		List<Service> list = serviceRepository.getOptimusInfo(serviceids);
    		if (list != null && !list.isEmpty()) {
    			list = inlineServiceIncidences(list);
    		}
    		return ResponseEntity.ok(list);
        } catch (Exception e) {
        	log.error("getOptimusInfo  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceGetOptimusInfoError", "Cannot get the Service Optimus Info")).body(null);
        }
    }
    
    
    /**
     * GET  /service-providers/cspservices : get all the servics by CSP.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of services in body
     */
    @GetMapping("/services/cspservices")
    @Timed
    @Transactional
    public ResponseEntity<List<Service>> findServicesByCSP(@RequestParam String cspname) {
       log.debug("REST request to get Services");
       try {
    	   log.debug("getCSPServices  ::  loading Optimus info...");
    	   List<Service> list = serviceRepository.findServicesByCSP(CSP_TYPE_ID, cspname);
    	   if (list != null && !list.isEmpty()) {
    		   list = inlineServiceIncidences(list);
    	   }
    	   return ResponseEntity.ok(list);
       	} catch (Exception e) {
       		log.error("getCSPServices  ::  Exception  ::  " + e.getMessage());
       		return ResponseEntity.badRequest()
                   .headers(HeaderUtil.createFailureAlert(
                   		ENTITY_NAME, "serviceGetAllError", "Cannot get the list of Services")).body(null);
       	}
    }
    

    /**
     * GET  /services/:id : get the "id" service.
     *
     * @param id the id of the service to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the service, or with status 404 (Not Found)
     */
    @GetMapping("/services/{id}")
    @Timed
    public ResponseEntity<Service> getService(@PathVariable Long id) {
    	log.debug("REST request to get Service : {}", id);
        try {
        	log.debug("getService  ::  getting Service...");
        	
        	Service service = null;
        	List<ServiceFullInfoVO> list = serviceRepository.findService(id);
        	if (list != null && !list.isEmpty()) {
        		ServiceFullInfoVO temp = list.get(0);
        		service = new Service(temp.getId(), temp.getName(), temp.getClassid(), temp.getClassname(), temp.getDeleteddate());
        		service.setFirstcontract(temp.getFirstcontract());
        		service.setSecondcontract(temp.getSecondcontract());
        		service.setThirdcontract(temp.getThirdcontract());
        		for (ServiceFullInfoVO vo : list) {        			
        			AttributeVO attr = buildAttributeVO(vo);
        			service.getAttributes().add(attr);
        		}
        		
        		// Obtener incidencias
        		ServiceIncidenceResource serviceIncidenceResource = new ServiceIncidenceResource(serviceIncidenceRepository); 
       			service.setIncidences(serviceIncidenceResource.findIncidencesByServiceId(id));
        	}
        	
        	log.debug("getService  ::  getting OK");
        	return ResponseUtil.wrapOrNotFound(Optional.ofNullable(service));
        } catch (Exception e) {
        	log.error("getService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceGetError", "No puede obtenerse el Servicio"))
                    .body(null);
        }
    }
    

    /**
     * DELETE  /services/:id : delete the "id" service.
     *
     * @param id the id of the service to delete
     * @return the ResponseEntity with status 200 (OK)
     */
    @DeleteMapping("/services/{id}")
    @Timed
    @Transactional
    public ResponseEntity<Void> deleteService(@PathVariable Long id, @RequestParam String forcedelete) {
    	log.debug("REST request to delete Service : {}", id);
        try {
        	log.debug("deleteService  ::  forcedelete=" + forcedelete);
        	
        	// Obtener el servicio
        	Service service = serviceRepository.findOne(id);
        	boolean markAsErased = false;
        	
        	if (forcedelete != null && !Boolean.valueOf(forcedelete).booleanValue()) {
        		// Marcar el servicio como borrado
            	if (service != null) {
    	        	log.debug("deleteService  ::  soft deleting Service...");
    	        	
    	        	// Marcar atributos del servicio como borrados
    	        	serviceAttributeValueRepository.updateDeletedDateByIdService(ZonedDateTime.now(), id);
    	        	
    	        	// Marcar Servicio como borrado
    	        	markAsErased = true;
    	        	service.setDeleteddate(ZonedDateTime.now());
    	        	serviceRepository.saveAndFlush(service);
            	}
        	} else {
        		// Eliminar definitivamente los atributos del servicio
        		serviceAttributeValueRepository.deleteByIdService(id);
        		
        		// Eliminar definitivamente el servicio
        		log.debug("deleteService  ::  deleting service...");
            	serviceRepository.delete(id);
        	}
        	log.debug("deleteService  ::  deleting OK");
        	
            return markAsErased
            		? ResponseEntity.ok().headers(HeaderUtil.createEntitySoftDeletionAlert(ENTITY_NAME, service.getServicename())).build()
            				: ResponseEntity.ok().headers(HeaderUtil.createEntityDeletionAlert(ENTITY_NAME, service.getServicename())).build();
        } catch (DataIntegrityViolationException e) {
        	log.error("deleteService DataIntegrityViolationException");
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceDeleteDataIntegrityError", "El Servicio no puede ser eliminado porque esta siendo utilizado en Contratos"))
                    .body(null);
        } catch (EmptyResultDataAccessException e) {
            return ResponseEntity.noContent().build();
        } catch (Exception e) {
        	log.error("deleteService  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceDeleteError", "El Servicio no puede ser eliminado"))
                    .body(null);
        }
    }
    
    
    /**
     * Converts an attribute in String format into an ServiceAttributeValue object
     * @param attribute The original attribute in String format
     * @return ServiceattributeValue
     * @throws IOException 
     * @throws JsonMappingException 
     * @throws JsonParseException
     */
    private ServiceAttributeValue convertAttributeStringToObject(final Long serviceid, final String attribute) throws JsonParseException, JsonMappingException, IOException {
    	log.debug("deleteSconvertAttributeStringToObjectervice  ::  converting String...");
    	ServiceAttributeValue object = null;
    	if (attribute != null && !attribute.isEmpty()) {
    		ObjectMapper mapper = new ObjectMapper();
    		AttributeVO vo = mapper.readValue(attribute, AttributeVO.class);
    		if (vo != null) {
    			object = new ServiceAttributeValue();
    			object.setServiceattributevalue(vo.getValue());
    			object.setServiceattributetypeid(vo.getTypeid());
    			object.setUnitvalue(vo.getUnitvalue());
    			object.setChecklistvalue(vo.getChecklistvalue());
    			object.setServiceid(serviceid);
    		}
    	}
    	log.debug("deleteSconvertAttributeStringToObjectervice  ::  converting OK...");
    	return object;
    }
    
    
    /**
     * Builds an AttributeVO object.
     * @param object ServiceFullInfoVO object.
     * @return AttributeVO
     */
    private AttributeVO buildAttributeVO(final ServiceFullInfoVO object) {
    	AttributeVO attr = new AttributeVO();
		attr.setTypeid(object.getAttributetypeid());
		attr.setName(object.getAttributename());
		attr.setNfrname(object.getAttributenfrname());
		attr.setIsenumeration(object.isIsenumeration());
		attr.setIschecklist(object.isIschecklist());
		attr.setIscommon(object.isIscommon());
		attr.setIsfunctionalrequirement(object.isIsfunctionalrequirement());
		attr.setParentid(object.getParentid());
		attr.setChildid(object.getChildid());
		attr.setUnits(object.getUnits());
		attr.setUnitfactor(object.getUnitfactor());
		attr.setUnitrule(object.getUnitrule());
		attr.setChecklistevalrule(object.getChecklistevalrule());
		attr.setUnitvalue(object.getUnitvalue());
		attr.setChecklistvalue(object.getChecklistvalue());
		attr.setValueid(object.getAttributevalueid());
		attr.setValue(object.getValue());
		attr.setHuman(object.getDefinition());
		return attr;
    }
    
    
    /**
     * Checks if the service exists for insert operation
     * @param name Name
     * @return boolean
     */
    private boolean checkIfServiceExistsForInsert(final String name) {
    	if (name != null && !name.isEmpty()) {
    		log.debug("checkIfServiceExistsForInsert  ::  finding Service...");
    		return serviceRepository.checkIfServiceExistsForInsert(name.trim()) != null;
    	}
    	return false;
    }
    
    
    /**
     * Checks if the service exists for update operation
     * @param identifier The identifier
     * @param name Name
     * @return boolean
     */
    private boolean checkIfServiceExistsForUpdate(final Long identifier, final String name) {
    	if (identifier != null && identifier.longValue() > 0
    			&& name != null && !name.isEmpty()) {
    		log.debug("checkIfServiceExistsForUpdate  ::  finding Service...");
    		return serviceRepository.checkIfServiceExistsForUpdate(identifier, name.trim()) != null;
    	}
    	return false;
    }
    
    
    /**
     * Build the service list with its incidences inline.
     * @param sourceList The source list.
     * @return List
     */
    private List<Service> inlineServiceIncidences(final List<Service> sourceList) {
    	Map<Long, Service> map = new HashMap<Long, Service>();
    	for (Service srv : sourceList) {
    		if (srv.getIncidences().get(0).getId() == null) {
    			srv.getIncidences().clear();
    		}
    		
    		if (!map.containsKey(srv.getId())) {
    			map.put(srv.getId(), srv);
    		}
    		else {
    			map.get(srv.getId()).getIncidences().add(srv.getIncidences().get(0));
    		}
    	}
    	
    	return new ArrayList<Service>(map.values());
    }
}
