/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.frontend.server.service;

import eu.decideh2020.acsmi.frontend.server.domain.Authority;
import eu.decideh2020.acsmi.frontend.server.domain.User;
import eu.decideh2020.acsmi.frontend.server.repository.AuthorityRepository;
import eu.decideh2020.acsmi.frontend.server.config.Constants;
import eu.decideh2020.acsmi.frontend.server.repository.UserRepository;
import eu.decideh2020.acsmi.frontend.server.security.AuthoritiesConstants;
import eu.decideh2020.acsmi.frontend.server.security.SecurityUtils;
import eu.decideh2020.acsmi.frontend.server.service.util.RandomUtil;
import eu.decideh2020.acsmi.frontend.server.web.rest.vm.ManagedUserVM;
import eu.decideh2020.acsmi.frontend.server.service.dto.UserDTO;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Service class for managing users.
 */
@Service
@Transactional
public class UserService {

    private final Logger log = LoggerFactory.getLogger(UserService.class);

    private final UserRepository userRepository;

    private final PasswordEncoder passwordEncoder;

    private final AuthorityRepository authorityRepository;

    private String ownerCompany;

    private String originalCreditCardType;

    private String originalCreditCard;

    private String originalContractReference;

    public UserService(UserRepository userRepository, PasswordEncoder passwordEncoder, AuthorityRepository authorityRepository) {
        this.userRepository = userRepository;
        this.passwordEncoder = passwordEncoder;
        this.authorityRepository = authorityRepository;
    }

    public Optional<User> activateRegistration(String key) {
        log.debug("Activating user for activation key {}", key);
        return userRepository.findOneByActivationKey(key)
            .map(user -> {
                // activate given user for the registration key.
                user.setActivated(true);
                user.setActivationKey(null);
                log.debug("Activated user: {}", user);
                return user;
            });
    }

    public Optional<User> completePasswordReset(String newPassword, String key) {
       log.debug("Reset user password for reset key {}", key);

       return userRepository.findOneByResetKey(key)
           .filter(user -> user.getResetDate().isAfter(Instant.now().minusSeconds(86400)))
           .map(user -> {
                user.setPassword(passwordEncoder.encode(newPassword));
                user.setResetKey(null);
                user.setResetDate(null);
                return user;
           });
    }

    public Optional<User> requestPasswordReset(String mail) {
        return userRepository.findOneByEmail(mail)
            .filter(User::getActivated)
            .map(user -> {
                user.setResetKey(RandomUtil.generateResetKey());
                user.setResetDate(Instant.now());
                return user;
            });
    }

    public User createUser(String login, String password, String firstName, String lastName, String email,
    	String company, String usersGroup, String userType, String creditCardType, String creditCard, String contractReference,
      String cifOrVat, String imageUrl, String langKey) {

        User newUser = new User();
        Authority authorityOwner = null;

        if(userType.equals("csp")) {
        	log.debug("ROLE CSP ADDED");
        	authorityOwner = authorityRepository.findOne(AuthoritiesConstants.CSP);
        } else if(userType.equals("dev")) {
        	log.debug("ROLE USER ADDED");
        	authorityOwner = authorityRepository.findOne(AuthoritiesConstants.USER);
        } else if(userType.equals("lgl")) {
        	log.debug("ROLE LEGAL EXPERT ADDED");
        	authorityOwner = authorityRepository.findOne(AuthoritiesConstants.LEGAL_EXPERT);
        } else {
        	log.debug("ROLE ADMIN ADDED");
        	authorityOwner = authorityRepository.findOne(AuthoritiesConstants.ADMIN);
        }
       
        Authority authorityBasic = authorityRepository.findOne(AuthoritiesConstants.USER);
        Set<Authority> authorities = new HashSet<>();

        String encryptedPassword = passwordEncoder.encode(password);
        newUser.setLogin(login);
        // new user gets initially a generated password
        newUser.setPassword(encryptedPassword);
        newUser.setFirstName(firstName);
        newUser.setLastName(lastName);
        newUser.setEmail(email);
        newUser.setCompany(company);
        newUser.setUsersGroup(usersGroup);
        newUser.setUserType(userType);
        newUser.setCreditCardType(creditCardType);
        newUser.setCreditCard(creditCard);
        newUser.setContractReference(contractReference);
        newUser.setCifOrVat(cifOrVat);
        newUser.setImageUrl(imageUrl);
        newUser.setLangKey(langKey);
        
        // new user is not active
        //newUser.setActivated(false);
        
        newUser.setActivated(true);
        
        // new user gets registration key
        //newUser.setActivationKey(RandomUtil.generateActivationKey());

        /*
        authorities.add(authorityBasic);
        if (authorityOwner != null) {
        	authorities.add(authorityOwner);
        }
        newUser.setAuthorities(authorities);
        */
        authorities.add(authorityOwner != null ? authorityOwner : authorityBasic);
        newUser.setAuthorities(authorities);

        userRepository.save(newUser);
        log.debug("Created Information for User: {}", newUser);
        return newUser;
    }

    public User createUser(ManagedUserVM userDTO) {
        User user = new User();
        user.setLogin(userDTO.getLogin());
        user.setFirstName(userDTO.getFirstName());
        user.setLastName(userDTO.getLastName());
        user.setEmail(userDTO.getEmail());
        user.setCompany(userDTO.getCompany());
        user.setUsersGroup(userDTO.getUsersGroup());
        user.setUserType(userDTO.getUserType());
        user.setCreditCardType(userDTO.getCreditCardType());
        user.setCreditCard(userDTO.getCreditCard());
        user.setContractReference(userDTO.getContractReference());
        user.setCifOrVat(userDTO.getCifOrVat());
        user.setImageUrl(userDTO.getImageUrl());
        if (userDTO.getLangKey() == null) {
            user.setLangKey("en"); // default language
        } else {
            user.setLangKey(userDTO.getLangKey());
        }
        if (userDTO.getAuthorities() != null) {
            Set<Authority> authorities = new HashSet<>();
            userDTO.getAuthorities().forEach(
                authority -> authorities.add(authorityRepository.findOne(authority))
            );
            user.setAuthorities(authorities);
        }
        
        //String encryptedPassword = passwordEncoder.encode(RandomUtil.generatePassword());
        String encryptedPassword = passwordEncoder.encode(userDTO.getPassword() != null ? userDTO.getPassword() : RandomUtil.generatePassword());
        user.setPassword(encryptedPassword);
        
        user.setResetKey(RandomUtil.generateResetKey());
        user.setResetDate(Instant.now());
        user.setActivated(true);
        userRepository.save(user);
        log.debug("Created Information for User: {}", user);
        return user;
    }

    /**
     * Update basic information (first name, last name, email, creditCardType, creditCard, contractReference, language) for the current user.
     *
     * @param firstName first name of user
     * @param lastName last name of user
     * @param email email id of user
     * @param company company name of user
     * @param creditCardType creditCardType of user
     * @param creditCard creditCard of user
     * @param contractReference contractReference of user
     * @param langKey language key
     * @param imageUrl image URL of user
     */
    public void updateUser(String firstName, String lastName, String email, String company, String creditCardType,
    		String creditCard, String contractReference, String langKey, String imageUrl) {
    	// Actualiza la info del usuario
    	userRepository.findOneByLogin(SecurityUtils.getCurrentUserLogin()).ifPresent(user -> {
            // Establece valores del usuario padre en variables para las siguientes comparaciones
    		ownerCompany = user.getCompany();
    		originalCreditCardType = user.getCreditCardType();
    		originalCreditCard = user.getCreditCard();
    		originalContractReference = user.getContractReference();

    		// Sustituye los valores antiguos por los nuevos
    		user.setFirstName(firstName);
    		user.setLastName(lastName);
            user.setEmail(email);
            user.setCreditCardType(creditCardType);
            user.setCreditCard(creditCard);
            user.setContractReference(contractReference);
            user.setLangKey(langKey);
            user.setImageUrl(imageUrl);
            log.debug("Changed Information for User: {}", user);
        });
    	/*
    	// Actualiza la informacion que haya cambiado a los demas usuarios que dependen de este
    	userRepository.findAllByCreditCardType(originalCreditCardType).forEach(user -> {
        	if(ownerCompany.equals(user.getCompany())) {
        		user.setCreditCardType(creditCardType);
                log.debug("Changed Credit Card Type for User: {}", user.getLogin());
        	} else {
        		log.debug("This user does not belong to the same company: {}", user.getLogin());
        	}
        });
    	*/
    	/*
    	userRepository.findAllByCreditCard(originalCreditCard).forEach(user -> {
    		if(ownerCompany.equals(user.getCompany())) {
    			user.setCreditCard(creditCard);
    			log.debug("Changed Credit Card Number for User: {}", user.getLogin());
        	} else {
        		log.debug("This user does not belong to the same company: {}", user.getLogin());
        	}
        });
    	*/
    	/*
        userRepository.findAllByContractReference(originalContractReference).forEach(user -> {
        	if(ownerCompany.equals(user.getCompany())) {
        		user.setContractReference(contractReference);
                log.debug("Changed Contract Reference for User: {}", user.getLogin());
        	} else {
        		log.debug("This user does not belong to the same company: {}", user.getLogin());
        	}
        });
        */
    }

    /**
     * Update all information for a specific user, and return the modified user.
     *
     * @param userDTO user to update
     * @return updated user
     */
    public Optional<UserDTO> updateUser(UserDTO userDTO) {
        return Optional.of(userRepository
            .findOne(userDTO.getId()))
            .map(user -> {
                user.setLogin(userDTO.getLogin());
                user.setFirstName(userDTO.getFirstName());
                user.setLastName(userDTO.getLastName());
                user.setEmail(userDTO.getEmail());
                user.setCompany(userDTO.getCompany());
                user.setUsersGroup(userDTO.getUsersGroup());
                user.setUserType(userDTO.getUserType());
                user.setCreditCardType(userDTO.getCreditCardType());
                user.setCreditCard(userDTO.getCreditCard());
                user.setContractReference(userDTO.getContractReference());
                user.setCifOrVat(userDTO.getCifOrVat());
                user.setImageUrl(userDTO.getImageUrl());
                user.setActivated(userDTO.isActivated());
                user.setLangKey(userDTO.getLangKey());
                Set<Authority> managedAuthorities = user.getAuthorities();
                managedAuthorities.clear();
                userDTO.getAuthorities().stream()
                    .map(authorityRepository::findOne)
                    .forEach(managedAuthorities::add);
                log.debug("Changed Information for User: {}", user);
                return user;
            })
            .map(UserDTO::new);
    }

    public void deleteUser(String login) {
        userRepository.findOneByLogin(login).ifPresent(user -> {
            userRepository.delete(user);
            log.debug("Deleted User: {}", user);
        });
    }

    public void changePassword(String password) {
        userRepository.findOneByLogin(SecurityUtils.getCurrentUserLogin()).ifPresent(user -> {
            String encryptedPassword = passwordEncoder.encode(password);
            user.setPassword(encryptedPassword);
            log.debug("Changed password for User: {}", user);
        });
    }

    @Transactional(readOnly = true)
    public Page<UserDTO> getAllManagedUsers(Pageable pageable) {
        return userRepository.findAllByLoginNot(pageable, Constants.ANONYMOUS_USER).map(UserDTO::new);
    }

    @Transactional(readOnly = true)
    public Optional<User> getUserWithAuthoritiesByLogin(String login) {
        return userRepository.findOneWithAuthoritiesByLogin(login);
    }

    @Transactional(readOnly = true)
    public User getUserWithAuthorities(Long id) {
        return userRepository.findOneWithAuthoritiesById(id);
    }

    @Transactional(readOnly = true)
    public User getUserWithAuthorities() {
        return userRepository.findOneWithAuthoritiesByLogin(SecurityUtils.getCurrentUserLogin()).orElse(null);
    }


    /**
     * Not activated users should be automatically deleted after 3 days.
     * <p>
     * This is scheduled to get fired everyday, at 01:00 (am).
     * </p>
     */
    @Scheduled(cron = "0 0 1 * * ?")
    public void removeNotActivatedUsers() {
        List<User> users = userRepository.findAllByActivatedIsFalseAndCreatedDateBefore(Instant.now().minus(3, ChronoUnit.DAYS));
        for (User user : users) {
            log.debug("Deleting not activated user {}", user.getLogin());
            userRepository.delete(user);
        }
    }

    /**
     * @return a list of all the authorities
     */
    public List<String> getAuthorities() {
        return authorityRepository.findAll().stream().map(Authority::getName).collect(Collectors.toList());
    }
}
